#ifndef winit_h
#define winit_h

#include <stdarg.h>
#include <stdbool.h>
#include <stddef.h>
#include <stdint.h>
#include <stdlib.h>


enum RawDisplayHandleType {
  RawDisplayHandleType_Unknown,
  RawDisplayHandleType_UiKit,
  RawDisplayHandleType_AppKit,
  RawDisplayHandleType_Orbital,
  RawDisplayHandleType_Xlib,
  RawDisplayHandleType_Xcb,
  RawDisplayHandleType_Wayland,
  RawDisplayHandleType_Drm,
  RawDisplayHandleType_Gbm,
  RawDisplayHandleType_Windows,
  RawDisplayHandleType_Web,
  RawDisplayHandleType_Android,
  RawDisplayHandleType_Haiku,
};
typedef uint8_t RawDisplayHandleType;

enum RawWindowHandleType {
  RawWindowHandleType_Unknown,
  RawWindowHandleType_UiKit,
  RawWindowHandleType_AppKit,
  RawWindowHandleType_Orbital,
  RawWindowHandleType_Xlib,
  RawWindowHandleType_Xcb,
  RawWindowHandleType_Wayland,
  RawWindowHandleType_Drm,
  RawWindowHandleType_Gbm,
  RawWindowHandleType_Win32,
  RawWindowHandleType_WinRt,
  RawWindowHandleType_Web,
  RawWindowHandleType_AndroidNdk,
  RawWindowHandleType_Haiku,
};
typedef uint8_t RawWindowHandleType;

/**
 * Symbolic name for a keyboard key.
 */
enum VirtualKeyCode {
  /**
   * The '1' key over the letters.
   */
  VirtualKeyCode_Key1,
  /**
   * The '2' key over the letters.
   */
  VirtualKeyCode_Key2,
  /**
   * The '3' key over the letters.
   */
  VirtualKeyCode_Key3,
  /**
   * The '4' key over the letters.
   */
  VirtualKeyCode_Key4,
  /**
   * The '5' key over the letters.
   */
  VirtualKeyCode_Key5,
  /**
   * The '6' key over the letters.
   */
  VirtualKeyCode_Key6,
  /**
   * The '7' key over the letters.
   */
  VirtualKeyCode_Key7,
  /**
   * The '8' key over the letters.
   */
  VirtualKeyCode_Key8,
  /**
   * The '9' key over the letters.
   */
  VirtualKeyCode_Key9,
  /**
   * The '0' key over the 'O' and 'P' keys.
   */
  VirtualKeyCode_Key0,
  VirtualKeyCode_A,
  VirtualKeyCode_B,
  VirtualKeyCode_C,
  VirtualKeyCode_D,
  VirtualKeyCode_E,
  VirtualKeyCode_F,
  VirtualKeyCode_G,
  VirtualKeyCode_H,
  VirtualKeyCode_I,
  VirtualKeyCode_J,
  VirtualKeyCode_K,
  VirtualKeyCode_L,
  VirtualKeyCode_M,
  VirtualKeyCode_N,
  VirtualKeyCode_O,
  VirtualKeyCode_P,
  VirtualKeyCode_Q,
  VirtualKeyCode_R,
  VirtualKeyCode_S,
  VirtualKeyCode_T,
  VirtualKeyCode_U,
  VirtualKeyCode_V,
  VirtualKeyCode_W,
  VirtualKeyCode_X,
  VirtualKeyCode_Y,
  VirtualKeyCode_Z,
  /**
   * The Escape key, next to F1.
   */
  VirtualKeyCode_Escape,
  VirtualKeyCode_F1,
  VirtualKeyCode_F2,
  VirtualKeyCode_F3,
  VirtualKeyCode_F4,
  VirtualKeyCode_F5,
  VirtualKeyCode_F6,
  VirtualKeyCode_F7,
  VirtualKeyCode_F8,
  VirtualKeyCode_F9,
  VirtualKeyCode_F10,
  VirtualKeyCode_F11,
  VirtualKeyCode_F12,
  VirtualKeyCode_F13,
  VirtualKeyCode_F14,
  VirtualKeyCode_F15,
  VirtualKeyCode_F16,
  VirtualKeyCode_F17,
  VirtualKeyCode_F18,
  VirtualKeyCode_F19,
  VirtualKeyCode_F20,
  VirtualKeyCode_F21,
  VirtualKeyCode_F22,
  VirtualKeyCode_F23,
  VirtualKeyCode_F24,
  /**
   * Print Screen/SysRq.
   */
  VirtualKeyCode_Snapshot,
  /**
   * Scroll Lock.
   */
  VirtualKeyCode_Scroll,
  /**
   * Pause/Break key, next to Scroll lock.
   */
  VirtualKeyCode_Pause,
  /**
   * `Insert`, next to Backspace.
   */
  VirtualKeyCode_Insert,
  VirtualKeyCode_Home,
  VirtualKeyCode_Delete,
  VirtualKeyCode_End,
  VirtualKeyCode_PageDown,
  VirtualKeyCode_PageUp,
  VirtualKeyCode_Left,
  VirtualKeyCode_Up,
  VirtualKeyCode_Right,
  VirtualKeyCode_Down,
  /**
   * The Backspace key, right over Enter.
   */
  VirtualKeyCode_Back,
  /**
   * The Enter key.
   */
  VirtualKeyCode_Return,
  /**
   * The space bar.
   */
  VirtualKeyCode_Space,
  /**
   * The "Compose" key on Linux.
   */
  VirtualKeyCode_Compose,
  VirtualKeyCode_Caret,
  VirtualKeyCode_Numlock,
  VirtualKeyCode_Numpad0,
  VirtualKeyCode_Numpad1,
  VirtualKeyCode_Numpad2,
  VirtualKeyCode_Numpad3,
  VirtualKeyCode_Numpad4,
  VirtualKeyCode_Numpad5,
  VirtualKeyCode_Numpad6,
  VirtualKeyCode_Numpad7,
  VirtualKeyCode_Numpad8,
  VirtualKeyCode_Numpad9,
  VirtualKeyCode_NumpadAdd,
  VirtualKeyCode_NumpadDivide,
  VirtualKeyCode_NumpadDecimal,
  VirtualKeyCode_NumpadComma,
  VirtualKeyCode_NumpadEnter,
  VirtualKeyCode_NumpadEquals,
  VirtualKeyCode_NumpadMultiply,
  VirtualKeyCode_NumpadSubtract,
  VirtualKeyCode_AbntC1,
  VirtualKeyCode_AbntC2,
  VirtualKeyCode_Apostrophe,
  VirtualKeyCode_Apps,
  VirtualKeyCode_Asterisk,
  VirtualKeyCode_At,
  VirtualKeyCode_Ax,
  VirtualKeyCode_Backslash,
  VirtualKeyCode_Calculator,
  VirtualKeyCode_Capital,
  VirtualKeyCode_Colon,
  VirtualKeyCode_Comma,
  VirtualKeyCode_Convert,
  VirtualKeyCode_Equals,
  VirtualKeyCode_Grave,
  VirtualKeyCode_Kana,
  VirtualKeyCode_Kanji,
  VirtualKeyCode_LAlt,
  VirtualKeyCode_LBracket,
  VirtualKeyCode_LControl,
  VirtualKeyCode_LShift,
  VirtualKeyCode_LWin,
  VirtualKeyCode_Mail,
  VirtualKeyCode_MediaSelect,
  VirtualKeyCode_MediaStop,
  VirtualKeyCode_Minus,
  VirtualKeyCode_Mute,
  VirtualKeyCode_MyComputer,
  VirtualKeyCode_NavigateForward,
  VirtualKeyCode_NavigateBackward,
  VirtualKeyCode_NextTrack,
  VirtualKeyCode_NoConvert,
  VirtualKeyCode_OEM102,
  VirtualKeyCode_Period,
  VirtualKeyCode_PlayPause,
  VirtualKeyCode_Plus,
  VirtualKeyCode_Power,
  VirtualKeyCode_PrevTrack,
  VirtualKeyCode_RAlt,
  VirtualKeyCode_RBracket,
  VirtualKeyCode_RControl,
  VirtualKeyCode_RShift,
  VirtualKeyCode_RWin,
  VirtualKeyCode_Semicolon,
  VirtualKeyCode_Slash,
  VirtualKeyCode_Sleep,
  VirtualKeyCode_Stop,
  VirtualKeyCode_Sysrq,
  VirtualKeyCode_Tab,
  VirtualKeyCode_Underline,
  VirtualKeyCode_Unlabeled,
  VirtualKeyCode_VolumeDown,
  VirtualKeyCode_VolumeUp,
  VirtualKeyCode_Wake,
  VirtualKeyCode_WebBack,
  VirtualKeyCode_WebFavorites,
  VirtualKeyCode_WebForward,
  VirtualKeyCode_WebHome,
  VirtualKeyCode_WebRefresh,
  VirtualKeyCode_WebSearch,
  VirtualKeyCode_WebStop,
  VirtualKeyCode_Yen,
  VirtualKeyCode_Copy,
  VirtualKeyCode_Paste,
  VirtualKeyCode_Cut,
};
typedef uint32_t VirtualKeyCode;

enum WinitControlFlow {
  /**
   * When the current loop iteration finishes, immediately begin a new iteration regardless of
   * whether or not new events are available to process.
   */
  WinitControlFlow_Poll,
  /**
   * When the current loop iteration finishes, suspend the thread until another event arrives.
   */
  WinitControlFlow_Wait,
  /**
   * Send a `LoopDestroyed` event and stop the event loop. This variant is *sticky* - once set,
   * `control_flow` cannot be changed from `Exit`, and any future attempts to do so will result
   * in the `control_flow` parameter being reset to `Exit`.
   */
  WinitControlFlow_Exit,
};
typedef uint32_t WinitControlFlow;

enum WinitCursorIcon {
  /**
   * The platform-dependent default cursor.
   */
  WinitCursorIcon_Default,
  /**
   * A simple crosshair.
   */
  WinitCursorIcon_Crosshair,
  /**
   * A hand (often used to indicate links in web browsers).
   */
  WinitCursorIcon_Hand,
  /**
   * Self explanatory.
   */
  WinitCursorIcon_Arrow,
  /**
   * Indicates something is to be moved.
   */
  WinitCursorIcon_Move,
  /**
   * Indicates text that may be selected or edited.
   */
  WinitCursorIcon_Text,
  /**
   * Program busy indicator.
   */
  WinitCursorIcon_Wait,
  /**
   * Help indicator (often rendered as a "?")
   */
  WinitCursorIcon_Help,
  /**
   * Progress indicator. Shows that processing is being done. But in contrast
   * with "Wait" the user may still interact with the program. Often rendered
   * as a spinning beach ball, or an arrow with a watch or hourglass.
   */
  WinitCursorIcon_Progress,
  /**
   * Cursor showing that something cannot be done.
   */
  WinitCursorIcon_NotAllowed,
  WinitCursorIcon_ContextMenu,
  WinitCursorIcon_Cell,
  WinitCursorIcon_VerticalText,
  WinitCursorIcon_Alias,
  WinitCursorIcon_Copy,
  WinitCursorIcon_NoDrop,
  WinitCursorIcon_Grab,
  WinitCursorIcon_Grabbing,
  WinitCursorIcon_AllScroll,
  WinitCursorIcon_ZoomIn,
  WinitCursorIcon_ZoomOut,
  /**
   * Indicate that some edge is to be moved. For example, the 'SeResize' cursor
   * is used when the movement starts from the south-east corner of the box.
   */
  WinitCursorIcon_EResize,
  WinitCursorIcon_NResize,
  WinitCursorIcon_NeResize,
  WinitCursorIcon_NwResize,
  WinitCursorIcon_SResize,
  WinitCursorIcon_SeResize,
  WinitCursorIcon_SwResize,
  WinitCursorIcon_WResize,
  WinitCursorIcon_EwResize,
  WinitCursorIcon_NsResize,
  WinitCursorIcon_NeswResize,
  WinitCursorIcon_NwseResize,
  WinitCursorIcon_ColResize,
  WinitCursorIcon_RowResize,
};
typedef uint32_t WinitCursorIcon;

enum WinitEventInputElementState {
  WinitEventInputElementState_Unknown,
  WinitEventInputElementState_Pressed,
  WinitEventInputElementState_Released,
};
typedef uint32_t WinitEventInputElementState;

enum WinitEventLoopType {
  WinitEventLoopType_Windows,
  WinitEventLoopType_MacOS,
  WinitEventLoopType_X11,
  WinitEventLoopType_Wayland,
  WinitEventLoopType_Unknown,
};
typedef uint8_t WinitEventLoopType;

enum WinitEventMouseButtonType {
  WinitEventMouseButtonType_Unknown,
  WinitEventMouseButtonType_Left,
  WinitEventMouseButtonType_Right,
  WinitEventMouseButtonType_Middle,
  WinitEventMouseButtonType_Other,
};
typedef uint32_t WinitEventMouseButtonType;

enum WinitEventMouseScrollDeltaType {
  WinitEventMouseScrollDeltaType_Unknown,
  WinitEventMouseScrollDeltaType_LineDelta,
  WinitEventMouseScrollDeltaType_PixelDelta,
};
typedef uint32_t WinitEventMouseScrollDeltaType;

enum WinitEventTouchPhase {
  WinitEventTouchPhase_Unknown,
  WinitEventTouchPhase_Started,
  WinitEventTouchPhase_Moved,
  WinitEventTouchPhase_Ended,
  WinitEventTouchPhase_Cancelled,
};
typedef uint32_t WinitEventTouchPhase;

enum WinitEventType {
  WinitEventType_Unknown,
  WinitEventType_WindowEventResized,
  WinitEventType_WindowEventMoved,
  WinitEventType_WindowEventCloseRequested,
  WinitEventType_WindowEventDestroyed,
  WinitEventType_WindowEventDroppedFile,
  WinitEventType_WindowEventHoveredFile,
  WinitEventType_WindowEventHoveredFileCancelled,
  WinitEventType_WindowEventReceivedCharacter,
  WinitEventType_WindowEventFocused,
  WinitEventType_WindowEventKeyboardInput,
  WinitEventType_WindowEventCursorMoved,
  WinitEventType_WindowEventCursorEntered,
  WinitEventType_WindowEventCursorLeft,
  WinitEventType_WindowEventMouseWheel,
  WinitEventType_WindowEventMouseInput,
  WinitEventType_WindowEventTouchpadPressure,
  WinitEventType_WindowEventAxisMotion,
  WinitEventType_WindowEventTouch,
  WinitEventType_WindowEventScaleFactorChanged,
  WinitEventType_NewEvents,
  WinitEventType_MainEventsCleared,
  WinitEventType_LoopDestroyed,
  WinitEventType_Suspended,
  WinitEventType_Resumed,
  WinitEventType_RedrawRequested,
  WinitEventType_RedrawEventsCleared,
  WinitEventType_ModifiersChanged,
  WinitEventType_UserEvent,
};
typedef uint32_t WinitEventType;

enum WinitUserEvent {
  /**
   * The virtual machine sends
   */
  WinitUserEvent_WakeUp,
};
typedef uint32_t WinitUserEvent;

/**
 * Used to send custom events to [`EventLoop`].
 */
typedef struct EventLoopProxy_WinitUserEvent EventLoopProxy_WinitUserEvent;

/**
 * Provides a way to retrieve events from the system and from the windows that were registered to
 * the events loop.
 *
 * An `EventLoop` can be seen more or less as a "context". Calling [`EventLoop::new`]
 * initializes everything that will be required to create windows. For example on Linux creating
 * an event loop opens a connection to the X or Wayland server.
 *
 * To wake up an `EventLoop` from a another thread, see the [`EventLoopProxy`] docs.
 *
 * Note that this cannot be shared across threads (due to platform-dependant logic
 * forbidding it), as such it is neither [`Send`] nor [`Sync`]. If you need cross-thread access, the
 * [`Window`] created from this _can_ be sent to an other thread, and the
 * [`EventLoopProxy`] allows you to wake up an `EventLoop` from another thread.
 *
 * [`Window`]: crate::window::Window
 */
typedef struct EventLoop_WinitUserEvent EventLoop_WinitUserEvent;

/**
 * Handle to a monitor.
 *
 * Allows you to retrieve information about a given monitor and can be used in [`Window`] creation.
 *
 * [`Window`]: crate::window::Window
 */
typedef struct MonitorHandle MonitorHandle;

typedef struct PhlowValue PhlowValue;

typedef struct PollingEventLoop PollingEventLoop;

typedef struct String String;

typedef struct Vec_u8 Vec_u8;

/**
 * The Win32 implementation of the main `Window` object.
 */
typedef struct Window Window;

typedef struct Window Window;

typedef struct Window Window;

typedef struct Window Window;

typedef struct Window Window;

typedef struct Window Window;

typedef struct Window Window;

typedef struct Window Window;

typedef struct Window Window;

/**
 * Object that allows building windows.
 */
typedef struct WindowBuilder WindowBuilder;

typedef struct WindowId WindowId;

typedef struct WindowId WindowId;

typedef struct WindowId WindowId;

typedef struct WindowId WindowId;

typedef struct WindowId WindowId;

typedef struct WindowId WindowId;

typedef struct WindowId WindowId;

typedef struct WindowRef WindowRef;

typedef struct WinitEventLoopWaker WinitEventLoopWaker;

typedef enum StringBoxOrigin_Tag {
  StringBoxOrigin_Byte,
  StringBoxOrigin_String,
  StringBoxOrigin_Wide,
} StringBoxOrigin_Tag;

typedef struct StringBoxOrigin {
  StringBoxOrigin_Tag tag;
  union {
    struct {
      struct Vec_u8 byte;
    };
    struct {
      U32String wide;
    };
  };
} StringBoxOrigin;

typedef struct String {
  struct StringBoxOrigin origin;
  struct String string;
} String;

enum StringBox_Tag {
  StringBox_Value_StringBox,
  StringBox_PhlowValue_StringBox,
};
typedef uint8_t StringBox_Tag;

typedef struct StringBox {
  StringBox_Tag tag;
  union {
    struct {
      struct String *value;
    };
    struct {
      struct PhlowValue *phlow_value;
    };
  };
} StringBox;

typedef struct EventLoop_WinitUserEvent WinitEventLoop;

enum EventLoop_Tag {
  EventLoop_Value_WinitEventLoop,
  EventLoop_PhlowValue_WinitEventLoop,
};
typedef uint8_t EventLoop_Tag;

typedef struct EventLoop {
  EventLoop_Tag tag;
  union {
    struct {
      WinitEventLoop *value;
    };
    struct {
      struct PhlowValue *phlow_value;
    };
  };
} EventLoop;

typedef struct U128 {
  uint64_t high;
  uint64_t low;
} U128;

typedef struct WinitTouchEvent {
  int64_t device_id;
  WinitEventTouchPhase phase;
  double x;
  double y;
  /**
   * unique identifier of a finger.
   */
  uint64_t id;
} WinitTouchEvent;

typedef struct WinitMouseScrollDelta {
  WinitEventMouseScrollDeltaType delta_type;
  double x;
  double y;
} WinitMouseScrollDelta;

typedef struct WinitMouseWheelEvent {
  int64_t device_id;
  WinitEventTouchPhase phase;
  struct WinitMouseScrollDelta delta;
} WinitMouseWheelEvent;

typedef struct WinitEventMouseButton {
  WinitEventMouseButtonType button_type;
  uint16_t button_code;
} WinitEventMouseButton;

typedef struct WinitMouseInputEvent {
  int64_t device_id;
  WinitEventInputElementState state;
  struct WinitEventMouseButton button;
} WinitMouseInputEvent;

typedef struct WinitCursorMovedEvent {
  int64_t device_id;
  double x;
  double y;
} WinitCursorMovedEvent;

typedef struct WinitEventKeyboardInput {
  int64_t device_id;
  uint32_t scan_code;
  WinitEventInputElementState state;
  bool has_virtual_keycode;
  VirtualKeyCode virtual_keycode;
  bool is_synthetic;
} WinitEventKeyboardInput;

typedef struct WinitEventReceivedCharacter {
  size_t length;
  uint8_t byte_1;
  uint8_t byte_2;
  uint8_t byte_3;
  uint8_t byte_4;
} WinitEventReceivedCharacter;

typedef struct WinitWindowResizedEvent {
  uint32_t width;
  uint32_t height;
} WinitWindowResizedEvent;

typedef struct WinitWindowScaleFactorChangedEvent {
  double scale_factor;
  uint32_t width;
  uint32_t height;
} WinitWindowScaleFactorChangedEvent;

typedef struct WinitWindowMovedEvent {
  int32_t x;
  int32_t y;
} WinitWindowMovedEvent;

typedef struct WinitWindowFocusedEvent {
  bool is_focused;
} WinitWindowFocusedEvent;

typedef struct WinitEventModifiersState {
  /**
   * The "shift" key
   */
  bool shift;
  /**
   * The "control" key
   */
  bool ctrl;
  /**
   * The "alt" key
   */
  bool alt;
  /**
   * The "logo" key
   *
   * This is the "windows" key on PC and "command" key on Mac.
   */
  bool logo;
} WinitEventModifiersState;

typedef struct WinitEventUserEvent {
  WinitUserEvent event;
} WinitEventUserEvent;

typedef struct WinitEvent {
  struct U128 window_id;
  WinitEventType event_type;
  struct WinitTouchEvent touch;
  struct WinitMouseWheelEvent mouse_wheel;
  struct WinitMouseInputEvent mouse_input;
  struct WinitCursorMovedEvent cursor_moved;
  struct WinitEventKeyboardInput keyboard_input;
  struct WinitEventReceivedCharacter received_character;
  struct WinitWindowResizedEvent window_resized;
  struct WinitWindowScaleFactorChangedEvent scale_factor;
  struct WinitWindowMovedEvent window_moved;
  struct WinitWindowFocusedEvent window_focused;
  struct WinitEventModifiersState modifiers;
  struct WinitEventUserEvent user_event;
} WinitEvent;

typedef struct EventLoopProxy_WinitUserEvent WinitEventLoopProxy;

enum ValueBox_WinitEventLoopProxy_Tag {
  ValueBox_WinitEventLoopProxy_Value_WinitEventLoopProxy,
  ValueBox_WinitEventLoopProxy_PhlowValue_WinitEventLoopProxy,
};
typedef uint8_t ValueBox_WinitEventLoopProxy_Tag;

typedef struct ValueBox_WinitEventLoopProxy {
  ValueBox_WinitEventLoopProxy_Tag tag;
  union {
    struct {
      WinitEventLoopProxy *value;
    };
    struct {
      struct PhlowValue *phlow_value;
    };
  };
} ValueBox_WinitEventLoopProxy;

enum ValueBox_MonitorHandle_Tag {
  ValueBox_MonitorHandle_Value_MonitorHandle,
  ValueBox_MonitorHandle_PhlowValue_MonitorHandle,
};
typedef uint8_t ValueBox_MonitorHandle_Tag;

typedef struct ValueBox_MonitorHandle {
  ValueBox_MonitorHandle_Tag tag;
  union {
    struct {
      struct MonitorHandle *value;
    };
    struct {
      struct PhlowValue *phlow_value;
    };
  };
} ValueBox_MonitorHandle;

enum ValueBox_WinitEventLoopWaker_Tag {
  ValueBox_WinitEventLoopWaker_Value_WinitEventLoopWaker,
  ValueBox_WinitEventLoopWaker_PhlowValue_WinitEventLoopWaker,
};
typedef uint8_t ValueBox_WinitEventLoopWaker_Tag;

typedef struct ValueBox_WinitEventLoopWaker {
  ValueBox_WinitEventLoopWaker_Tag tag;
  union {
    struct {
      struct WinitEventLoopWaker *value;
    };
    struct {
      struct PhlowValue *phlow_value;
    };
  };
} ValueBox_WinitEventLoopWaker;

enum ValueBox_PollingEventLoop_Tag {
  ValueBox_PollingEventLoop_Value_PollingEventLoop,
  ValueBox_PollingEventLoop_PhlowValue_PollingEventLoop,
};
typedef uint8_t ValueBox_PollingEventLoop_Tag;

typedef struct ValueBox_PollingEventLoop {
  ValueBox_PollingEventLoop_Tag tag;
  union {
    struct {
      struct PollingEventLoop *value;
    };
    struct {
      struct PhlowValue *phlow_value;
    };
  };
} ValueBox_PollingEventLoop;

enum ValueBox_WindowRef_Tag {
  ValueBox_WindowRef_Value_WindowRef,
  ValueBox_WindowRef_PhlowValue_WindowRef,
};
typedef uint8_t ValueBox_WindowRef_Tag;

typedef struct ValueBox_WindowRef {
  ValueBox_WindowRef_Tag tag;
  union {
    struct {
      struct WindowRef *value;
    };
    struct {
      struct PhlowValue *phlow_value;
    };
  };
} ValueBox_WindowRef;

enum WindowBuilder_Tag {
  WindowBuilder_Value_WindowBuilder,
  WindowBuilder_PhlowValue_WindowBuilder,
};
typedef uint8_t WindowBuilder_Tag;

typedef struct WindowBuilder {
  WindowBuilder_Tag tag;
  union {
    struct {
      struct WindowBuilder *value;
    };
    struct {
      struct PhlowValue *phlow_value;
    };
  };
} WindowBuilder;

enum ValueBox_WindowId_Tag {
  ValueBox_WindowId_Value_WindowId,
  ValueBox_WindowId_PhlowValue_WindowId,
};
typedef uint8_t ValueBox_WindowId_Tag;

typedef struct ValueBox_WindowId {
  ValueBox_WindowId_Tag tag;
  union {
    struct {
      struct WindowId *value;
    };
    struct {
      struct PhlowValue *phlow_value;
    };
  };
} ValueBox_WindowId;

typedef struct RawWindowHandle {
  RawWindowHandleType handle_type;
  void *ptr_1;
  void *ptr_2;
  void *ptr_3;
  uint64_t id_1;
  uint64_t id_2;
} RawWindowHandle;

typedef struct RawDisplayHandle {
  RawDisplayHandleType handle_type;
  void *ptr_1;
  uint64_t id_1;
} RawDisplayHandle;

typedef struct SizeU32 {
  uint32_t width;
  uint32_t height;
} SizeU32;

enum ValueBox_SizeBox_u32_Tag {
  ValueBox_SizeBox_u32_Value_SizeBox_u32,
  ValueBox_SizeBox_u32_PhlowValue_SizeBox_u32,
};
typedef uint8_t ValueBox_SizeBox_u32_Tag;

typedef struct ValueBox_SizeBox_u32 {
  ValueBox_SizeBox_u32_Tag tag;
  union {
    struct {
      struct SizeU32 *value;
    };
    struct {
      struct PhlowValue *phlow_value;
    };
  };
} ValueBox_SizeBox_u32;

typedef struct PointBox_i32 {
  int32_t x;
  int32_t y;
} PointBox_i32;

enum ValueBox_PointBox_i32_Tag {
  ValueBox_PointBox_i32_Value_PointBox_i32,
  ValueBox_PointBox_i32_PhlowValue_PointBox_i32,
};
typedef uint8_t ValueBox_PointBox_i32_Tag;

typedef struct ValueBox_PointBox_i32 {
  ValueBox_PointBox_i32_Tag tag;
  union {
    struct {
      struct PointBox_i32 *value;
    };
    struct {
      struct PhlowValue *phlow_value;
    };
  };
} ValueBox_PointBox_i32;

enum U128Box_Tag {
  U128Box_Value_U128Box,
  U128Box_PhlowValue_U128Box,
};
typedef uint8_t U128Box_Tag;

typedef struct U128Box {
  U128Box_Tag tag;
  union {
    struct {
      struct U128 *value;
    };
    struct {
      struct PhlowValue *phlow_value;
    };
  };
} U128Box;

enum Window_Tag {
  Window_Value_Window,
  Window_PhlowValue_Window,
};
typedef uint8_t Window_Tag;

typedef struct Window {
  Window_Tag tag;
  union {
    struct {
      struct Window *value;
    };
    struct {
      struct PhlowValue *phlow_value;
    };
  };
} Window;

enum ValueBox_ArrayBox_u8_Tag {
  ValueBox_ArrayBox_u8_Value_ArrayBox_u8,
  ValueBox_ArrayBox_u8_PhlowValue_ArrayBox_u8,
};
typedef uint8_t ValueBox_ArrayBox_u8_Tag;

typedef struct ValueBox_ArrayBox_u8 {
  ValueBox_ArrayBox_u8_Tag tag;
  union {
    struct {
      ArrayBox<uint8_t> *value;
    };
    struct {
      struct PhlowValue *phlow_value;
    };
  };
} ValueBox_ArrayBox_u8;

typedef struct Point3Box_f32 {
  float x;
  float y;
  float z;
} Point3Box_f32;

typedef struct Point3Box_f32 BoxerPoint3F32;

enum ValueBox_BoxerPoint3F32_Tag {
  ValueBox_BoxerPoint3F32_Value_BoxerPoint3F32,
  ValueBox_BoxerPoint3F32_PhlowValue_BoxerPoint3F32,
};
typedef uint8_t ValueBox_BoxerPoint3F32_Tag;

typedef struct ValueBox_BoxerPoint3F32 {
  ValueBox_BoxerPoint3F32_Tag tag;
  union {
    struct {
      BoxerPoint3F32 *value;
    };
    struct {
      struct PhlowValue *phlow_value;
    };
  };
} ValueBox_BoxerPoint3F32;

typedef struct PointF32 {
  float x;
  float y;
} PointF32;

typedef struct PointF32 PointF32;

enum PointBoxF32_Tag {
  PointBoxF32_Value_BoxerPointF32,
  PointBoxF32_PhlowValue_BoxerPointF32,
};
typedef uint8_t PointBoxF32_Tag;

typedef struct PointBoxF32 {
  PointBoxF32_Tag tag;
  union {
    struct {
      PointF32 *value;
    };
    struct {
      struct PhlowValue *phlow_value;
    };
  };
} PointBoxF32;

typedef struct PointBox_f64 {
  double x;
  double y;
} PointBox_f64;

typedef struct PointBox_f64 BoxerPointF64;

enum ValueBox_BoxerPointF64_Tag {
  ValueBox_BoxerPointF64_Value_BoxerPointF64,
  ValueBox_BoxerPointF64_PhlowValue_BoxerPointF64,
};
typedef uint8_t ValueBox_BoxerPointF64_Tag;

typedef struct ValueBox_BoxerPointF64 {
  ValueBox_BoxerPointF64_Tag tag;
  union {
    struct {
      BoxerPointF64 *value;
    };
    struct {
      struct PhlowValue *phlow_value;
    };
  };
} ValueBox_BoxerPointF64;

typedef struct PointBox_i32 BoxerPointI32;

enum ValueBox_BoxerPointI32_Tag {
  ValueBox_BoxerPointI32_Value_BoxerPointI32,
  ValueBox_BoxerPointI32_PhlowValue_BoxerPointI32,
};
typedef uint8_t ValueBox_BoxerPointI32_Tag;

typedef struct ValueBox_BoxerPointI32 {
  ValueBox_BoxerPointI32_Tag tag;
  union {
    struct {
      BoxerPointI32 *value;
    };
    struct {
      struct PhlowValue *phlow_value;
    };
  };
} ValueBox_BoxerPointI32;

typedef struct PointBox_u64 {
  uint64_t x;
  uint64_t y;
} PointBox_u64;

typedef struct PointBox_u64 BoxerPointU64;

enum ValueBox_BoxerPointU64_Tag {
  ValueBox_BoxerPointU64_Value_BoxerPointU64,
  ValueBox_BoxerPointU64_PhlowValue_BoxerPointU64,
};
typedef uint8_t ValueBox_BoxerPointU64_Tag;

typedef struct ValueBox_BoxerPointU64 {
  ValueBox_BoxerPointU64_Tag tag;
  union {
    struct {
      BoxerPointU64 *value;
    };
    struct {
      struct PhlowValue *phlow_value;
    };
  };
} ValueBox_BoxerPointU64;

enum ValueBox_Range_usize_Tag {
  ValueBox_Range_usize_Value_Range_usize,
  ValueBox_Range_usize_PhlowValue_Range_usize,
};
typedef uint8_t ValueBox_Range_usize_Tag;

typedef struct ValueBox_Range_usize {
  ValueBox_Range_usize_Tag tag;
  union {
    struct {
      Range<size_t> *value;
    };
    struct {
      struct PhlowValue *phlow_value;
    };
  };
} ValueBox_Range_usize;

typedef struct SizeF32 {
  float width;
  float height;
} SizeF32;

typedef struct SizeF32 SizeF32;

enum SizeBoxF32_Tag {
  SizeBoxF32_Value_BoxerSizeF32,
  SizeBoxF32_PhlowValue_BoxerSizeF32,
};
typedef uint8_t SizeBoxF32_Tag;

typedef struct SizeBoxF32 {
  SizeBoxF32_Tag tag;
  union {
    struct {
      SizeF32 *value;
    };
    struct {
      struct PhlowValue *phlow_value;
    };
  };
} SizeBoxF32;

typedef struct SizeF64 {
  double width;
  double height;
} SizeF64;

typedef struct SizeF64 SizeF64;

enum SizeBoxF64_Tag {
  SizeBoxF64_Value_BoxerSizeF64,
  SizeBoxF64_PhlowValue_BoxerSizeF64,
};
typedef uint8_t SizeBoxF64_Tag;

typedef struct SizeBoxF64 {
  SizeBoxF64_Tag tag;
  union {
    struct {
      SizeF64 *value;
    };
    struct {
      struct PhlowValue *phlow_value;
    };
  };
} SizeBoxF64;

typedef struct SizeI32 {
  int32_t width;
  int32_t height;
} SizeI32;

typedef struct SizeI32 SizeI32;

enum SizeBoxI32_Tag {
  SizeBoxI32_Value_BoxerSizeI32,
  SizeBoxI32_PhlowValue_BoxerSizeI32,
};
typedef uint8_t SizeBoxI32_Tag;

typedef struct SizeBoxI32 {
  SizeBoxI32_Tag tag;
  union {
    struct {
      SizeI32 *value;
    };
    struct {
      struct PhlowValue *phlow_value;
    };
  };
} SizeBoxI32;

typedef struct SizeU32 SizeU32;

enum SizeBoxU32_Tag {
  SizeBoxU32_Value_BoxerSizeU32,
  SizeBoxU32_PhlowValue_BoxerSizeU32,
};
typedef uint8_t SizeBoxU32_Tag;

typedef struct SizeBoxU32 {
  SizeBoxU32_Tag tag;
  union {
    struct {
      SizeU32 *value;
    };
    struct {
      struct PhlowValue *phlow_value;
    };
  };
} SizeBoxU32;

typedef struct SizeU64 {
  uint64_t width;
  uint64_t height;
} SizeU64;

typedef struct SizeU64 SizeU64;

enum SizeBoxU64_Tag {
  SizeBoxU64_Value_BoxerSizeU64,
  SizeBoxU64_PhlowValue_BoxerSizeU64,
};
typedef uint8_t SizeBoxU64_Tag;

typedef struct SizeBoxU64 {
  SizeBoxU64_Tag tag;
  union {
    struct {
      SizeU64 *value;
    };
    struct {
      struct PhlowValue *phlow_value;
    };
  };
} SizeBoxU64;

enum ValueBox_c_void_Tag {
  ValueBox_c_void_Value_c_void,
  ValueBox_c_void_PhlowValue_c_void,
};
typedef uint8_t ValueBox_c_void_Tag;

typedef struct ValueBox_c_void {
  ValueBox_c_void_Tag tag;
  union {
    struct {
      void *value;
    };
    struct {
      struct PhlowValue *phlow_value;
    };
  };
} ValueBox_c_void;

bool winit_test(void);

void winit_init_logger(void);

void winit_println(struct StringBox *message);

void winit_print(struct StringBox *message);

struct EventLoop *winit_event_loop_new(void);

void winit_event_loop_drop(struct EventLoop *_ptr);

WinitEventLoopType winit_event_loop_get_type(struct EventLoop *event_loop);

/**
 * Hijacks the calling thread and initializes the winit event loop with the provided closure.
 * Since the closure is 'static, it must be a move closure if it needs to access any data from the calling context.
 * See the ControlFlow docs for information on how changes to &mut ControlFlow impact the event loop's behavior.
 * Any values not passed to this function will not be dropped.
 */
void winit_event_loop_run_data(struct EventLoop *event_loop,
                               void *data,
                               WinitControlFlow (*callback)(void*, struct WinitEvent*));

struct ValueBox_WinitEventLoopProxy *winit_event_loop_create_proxy(struct EventLoop *event_loop);

void winit_event_loop_drop_proxy(struct ValueBox_WinitEventLoopProxy *event_loop_proxy);

struct ValueBox_MonitorHandle *winit_event_loop_get_primary_monitor(struct EventLoop *event_loop);

double winit_primary_monitor_get_hidpi_factor(struct ValueBox_MonitorHandle *monitor_handle);

void winit_primary_monitor_drop(struct ValueBox_MonitorHandle *ptr);

void winit_event_loop_run_return(struct EventLoop *event_loop,
                                 WinitControlFlow (*callback)(struct WinitEvent*));

/**
 * Initializes the winit event loop.
 * Unlike EventLoop::run, this function accepts non-'static (i.e. non-move) closures
 * and returns control flow to the caller when control_flow is set to ControlFlow::Exit.
 */
void winit_event_loop_run_return_data(struct EventLoop *event_loop,
                                      void *data,
                                      WinitControlFlow (*callback)(void*, struct WinitEvent*));

void winit_event_drop(struct WinitEvent *ptr);

bool winit_waker_wake(const void *waker, WinitUserEvent event);

struct ValueBox_WinitEventLoopWaker *winit_event_loop_waker_create(struct ValueBox_PollingEventLoop *event_loop);

bool (*winit_event_loop_waker_function(void))(const void*, WinitUserEvent);

void winit_event_loop_waker_drop(struct ValueBox_WinitEventLoopWaker *event_loop_waker);

struct ValueBox_PollingEventLoop *winit_polling_event_loop_new(void);

bool winit_polling_event_loop_wake(struct ValueBox_PollingEventLoop *event_loop,
                                   WinitUserEvent event);

struct ValueBox_WindowRef *winit_polling_event_loop_create_window(struct ValueBox_PollingEventLoop *event_loop,
                                                                  struct WindowBuilder *window_builder);

struct ValueBox_PollingEventLoop *winit_polling_event_loop_new_with_semaphore_and_main_events_signaller(void (*semaphore_callback)(size_t, const void*),
                                                                                                        size_t semaphore_index,
                                                                                                        const void *semaphore_thunk,
                                                                                                        void (*main_events_callback)(const void*),
                                                                                                        const void *main_events_thunk);

void winit_polling_event_loop_add_resize_listener(struct ValueBox_PollingEventLoop *event_loop,
                                                  struct ValueBox_WindowId *window_id,
                                                  void (*callback)(const void*, uint32_t, uint32_t),
                                                  const void *thunk);

size_t winit_polling_event_loop_count_resize_listeners(struct ValueBox_PollingEventLoop *event_loop);

void winit_polling_event_loop_add_redraw_listener(struct ValueBox_PollingEventLoop *event_loop,
                                                  struct ValueBox_WindowId *window_id,
                                                  void (*callback)(const void*),
                                                  const void *thunk);

size_t winit_polling_event_loop_count_redraw_listeners(struct ValueBox_PollingEventLoop *event_loop);

void winit_polling_event_loop_set_android_app(struct ValueBox_PollingEventLoop *event_loop,
                                              AndroidApp *android_app);

struct WinitEvent *winit_polling_event_loop_poll(struct ValueBox_PollingEventLoop *event_loop);

void winit_polling_event_loop_run(struct ValueBox_PollingEventLoop *event_loop);

/**
 * Must be called from the inside of the `run` method of the [`PollingEventLoop`].
 */
WinitEventLoopType winit_polling_event_loop_get_type(struct ValueBox_PollingEventLoop *event_loop);

void winit_polling_event_loop_drop(struct ValueBox_PollingEventLoop *event_loop);

/**
 * Return the raw window handle that can be used to create a native rendering context.
 * Must only be called from the main thread
 */
struct RawWindowHandle *winit_window_ref_raw_window_handle(struct ValueBox_PollingEventLoop *event_loop,
                                                           struct ValueBox_WindowRef *window_ref);

/**
 * Return the raw window handle that can be used to create a native rendering context.
 * Must only be called from the main thread
 */
struct RawDisplayHandle *winit_window_ref_raw_display_handle(struct ValueBox_PollingEventLoop *event_loop,
                                                             struct ValueBox_WindowRef *window_ref);

/**
 * Request the window to redraw. Can be called from any thread.
 */
void winit_window_ref_request_redraw(struct ValueBox_PollingEventLoop *event_loop,
                                     struct ValueBox_WindowRef *window_ref);

/**
 * Get the scaled factor of the window. Can be called from the any thread.
 */
double winit_window_ref_get_scale_factor(struct ValueBox_WindowRef *window_ref);

/**
 * Get the inner size of the window. Can be called from the any thread.
 */
void winit_window_ref_get_inner_size(struct ValueBox_WindowRef *window_ref,
                                     struct ValueBox_SizeBox_u32 *inner_size);

/**
 * Set the inner size of the window.
 * Must be called from the main thread
 */
void winit_window_ref_set_inner_size(struct ValueBox_PollingEventLoop *event_loop,
                                     struct ValueBox_WindowRef *window_ref,
                                     uint32_t width,
                                     uint32_t height);

/**
 * Get the outer position of the window. Can be called from the any thread.
 */
void winit_window_ref_get_position(struct ValueBox_WindowRef *window_ref,
                                   struct ValueBox_PointBox_i32 *position);

/**
 * Set the outer position of the window.
 * Must be called from the main thread
 */
void winit_window_ref_set_position(struct ValueBox_PollingEventLoop *event_loop,
                                   struct ValueBox_WindowRef *window_ref,
                                   int32_t x,
                                   int32_t y);

void winit_window_ref_get_id(struct ValueBox_WindowRef *window_ref, struct U128Box *id);

struct ValueBox_WindowId *winit_window_ref_get_raw_id(struct ValueBox_WindowRef *window_ref);

void winit_window_ref_set_title(struct ValueBox_PollingEventLoop *event_loop,
                                struct ValueBox_WindowRef *window_ref,
                                struct StringBox *title);

void winit_window_ref_set_cursor_icon(struct ValueBox_PollingEventLoop *event_loop,
                                      struct ValueBox_WindowRef *window_ref,
                                      WinitCursorIcon cursor_icon);

void winit_window_ref_set_maximized(struct ValueBox_PollingEventLoop *event_loop,
                                    struct ValueBox_WindowRef *window_ref,
                                    bool maximized);

void *winit_window_ref_get_hwnd(struct ValueBox_PollingEventLoop *event_loop,
                                struct ValueBox_WindowRef *window_ref);

void *winit_window_ref_get_xlib_display(struct ValueBox_PollingEventLoop *event_loop,
                                        struct ValueBox_WindowRef *window_ref);

unsigned long winit_window_ref_get_xlib_window(struct ValueBox_PollingEventLoop *event_loop,
                                               struct ValueBox_WindowRef *window_ref);

void *winit_window_ref_get_wayland_surface(struct ValueBox_PollingEventLoop *event_loop,
                                           struct ValueBox_WindowRef *window_ref);

void *winit_window_ref_get_wayland_display(struct ValueBox_PollingEventLoop *event_loop,
                                           struct ValueBox_WindowRef *window_ref);

void winit_window_ref_destroy(struct ValueBox_PollingEventLoop *event_loop,
                              struct ValueBox_WindowRef *window_ref);

struct Window *winit_create_window(struct EventLoop *event_loop,
                                   struct WindowBuilder *window_builder);

struct RawWindowHandle *winit_window_raw_window_handle(struct Window *window);

struct RawDisplayHandle *winit_window_raw_display_handle(struct Window *window);

void winit_window_request_redraw(struct Window *window);

double winit_window_get_scale_factor(struct Window *window);

void winit_window_get_inner_size(struct Window *window, struct ValueBox_SizeBox_u32 *size);

void winit_window_set_inner_size(struct Window *window, uint32_t width, uint32_t height);

void winit_window_get_position(struct Window *window, struct ValueBox_PointBox_i32 *position);

void winit_window_set_position(struct Window *window, int32_t x, int32_t y);

void winit_window_get_id(struct Window *window, struct U128Box *window_id);

void winit_window_set_title(struct Window *window, struct StringBox *window_title);

void winit_window_set_cursor_icon(struct Window *window, WinitCursorIcon cursor_icon);

void winit_window_set_maximized(struct Window *window, bool maximized);

void *winit_window_get_hwnd(struct Window *window_ptr);

void *winit_window_get_xlib_display(struct Window *window);

unsigned long winit_window_get_xlib_window(struct Window *window);

void winit_window_drop(struct Window *ptr);

struct WindowBuilder *winit_window_builder_new(void);

void winit_window_builder_drop(struct WindowBuilder *window_builder);

void winit_window_builder_with_title(struct WindowBuilder *window_builder,
                                     struct StringBox *window_title);

void winit_window_builder_with_decorations(struct WindowBuilder *window_builder,
                                           bool with_decorations);

void winit_window_builder_with_transparency(struct WindowBuilder *window_builder,
                                            bool with_transparency);

void winit_window_builder_with_resizable(struct WindowBuilder *window_builder, bool with_resizable);

void winit_window_builder_with_dimensions(struct WindowBuilder *window_builder,
                                          double width,
                                          double height);

void winit_window_builder_with_maximized(struct WindowBuilder *window_builder, bool with_maximized);

void winit_window_builder_with_visibility(struct WindowBuilder *window_builder,
                                          bool with_visibility);

void winit_window_builder_with_always_on_top(struct WindowBuilder *window_builder,
                                             bool with_always_on_top);

void winit_window_builder_with_full_size(struct WindowBuilder *_ptr_window_builder,
                                         bool _with_full_size);

void winit_window_builder_with_full_size(struct WindowBuilder *window_builder, bool with_full_size);

void very_raw_window_handle_drop(struct RawWindowHandle *pointer);

void very_raw_display_handle_drop(struct RawDisplayHandle *pointer);

bool boxer_test(void);

/**
 * In-place convert argb to rgba
 */
void boxer_array_u8_argb_to_rgba(struct ValueBox_ArrayBox_u8 *array);

/**
 * In-place convert bgra to argb
 */
void boxer_array_u8_bgra_to_argb(struct ValueBox_ArrayBox_u8 *array);

/**
 * In-place convert rgba to argb
 */
void boxer_array_u8_rgba_to_argb(struct ValueBox_ArrayBox_u8 *array);

struct U128Box *boxer_number_uint128_create(void);

void boxer_number_uint128_drop(struct U128Box *ptr);

uint64_t boxer_number_uint128_get_low(struct U128Box *number);

void boxer_number_uint128_set_low(struct U128Box *number, uint64_t low);

uint64_t boxer_number_uint128_get_high(struct U128Box *number);

void boxer_number_uint128_set_high(struct U128Box *number, uint64_t high);

void boxer_number_uint128_set_max(struct U128Box *number);

void boxer_number_uint128_set_min(struct U128Box *number);

struct ValueBox_BoxerPoint3F32 *boxer_point3_f32_default(void);

struct ValueBox_BoxerPoint3F32 *boxer_point3_f32_create(float x, float y, float z);

void boxer_point3_f32_drop(struct ValueBox_BoxerPoint3F32 *ptr);

float boxer_point3_f32_get_x(struct ValueBox_BoxerPoint3F32 *_point_ptr);

void boxer_point3_f32_set_x(struct ValueBox_BoxerPoint3F32 *_point_ptr, float x);

float boxer_point3_f32_get_y(struct ValueBox_BoxerPoint3F32 *_point_ptr);

void boxer_point3_f32_set_y(struct ValueBox_BoxerPoint3F32 *_point_ptr, float y);

float boxer_point3_f32_get_z(struct ValueBox_BoxerPoint3F32 *_point_ptr);

void boxer_point3_f32_set_z(struct ValueBox_BoxerPoint3F32 *_point_ptr, float z);

struct PointBoxF32 *boxer_point_f32_default(void);

struct PointBoxF32 *boxer_point_f32_create(float x, float y);

void boxer_point_f32_drop(struct PointBoxF32 *ptr);

float boxer_point_f32_get_x(struct PointBoxF32 *_point_ptr);

void boxer_point_f32_set_x(struct PointBoxF32 *_point_ptr, float x);

float boxer_point_f32_get_y(struct PointBoxF32 *_point_ptr);

void boxer_point_f32_set_y(struct PointBoxF32 *_point_ptr, float y);

struct ValueBox_BoxerPointF64 *boxer_point_f64_default(void);

struct ValueBox_BoxerPointF64 *boxer_point_f64_create(double x, double y);

void boxer_point_f64_drop(struct ValueBox_BoxerPointF64 *ptr);

double boxer_point_f64_get_x(struct ValueBox_BoxerPointF64 *_point_ptr);

void boxer_point_f64_set_x(struct ValueBox_BoxerPointF64 *_point_ptr, double x);

double boxer_point_f64_get_y(struct ValueBox_BoxerPointF64 *_point_ptr);

void boxer_point_f64_set_y(struct ValueBox_BoxerPointF64 *_point_ptr, double y);

struct ValueBox_BoxerPointI32 *boxer_point_i32_default(void);

struct ValueBox_BoxerPointI32 *boxer_point_i32_create(int32_t x, int32_t y);

void boxer_point_i32_drop(struct ValueBox_BoxerPointI32 *ptr);

int32_t boxer_point_i32_get_x(struct ValueBox_BoxerPointI32 *point_ptr);

void boxer_point_i32_set_x(struct ValueBox_BoxerPointI32 *point_ptr, int32_t x);

int32_t boxer_point_i32_get_y(struct ValueBox_BoxerPointI32 *point_ptr);

void boxer_point_i32_set_y(struct ValueBox_BoxerPointI32 *_point_ptr, int32_t y);

struct ValueBox_BoxerPointU64 *boxer_point_u64_default(void);

struct ValueBox_BoxerPointU64 *boxer_point_u64_create(uint64_t x, uint64_t y);

void boxer_point_u64_drop(struct ValueBox_BoxerPointU64 *ptr);

uint64_t boxer_point_u64_get_x(struct ValueBox_BoxerPointU64 *_point_ptr);

void boxer_point_u64_set_x(struct ValueBox_BoxerPointU64 *_point_ptr, uint64_t x);

uint64_t boxer_point_u64_get_y(struct ValueBox_BoxerPointU64 *_point_ptr);

void boxer_point_u64_set_y(struct ValueBox_BoxerPointU64 *_point_ptr, uint64_t y);

struct ValueBox_Range_usize *boxer_range_usize_create(void);

void boxer_range_usize_drop(struct ValueBox_Range_usize *range);

size_t boxer_range_usize_get_start(struct ValueBox_Range_usize *range);

void boxer_range_usize_set_start(struct ValueBox_Range_usize *range, size_t start);

size_t boxer_range_usize_get_end(struct ValueBox_Range_usize *range);

void boxer_range_usize_set_end(struct ValueBox_Range_usize *range, size_t end);

struct SizeBoxF32 *boxer_size_f32_create(void);

void boxer_size_f32_drop(struct SizeBoxF32 *ptr);

float boxer_size_f32_get_width(struct SizeBoxF32 *ptr);

void boxer_size_f32_set_width(struct SizeBoxF32 *ptr, float width);

float boxer_size_f32_get_height(struct SizeBoxF32 *ptr);

void boxer_size_f32_set_height(struct SizeBoxF32 *ptr, float height);

struct SizeBoxF64 *boxer_size_f64_create(void);

void boxer_size_f64_drop(struct SizeBoxF64 *ptr);

double boxer_size_f64_get_width(struct SizeBoxF64 *ptr);

void boxer_size_f64_set_width(struct SizeBoxF64 *ptr, double width);

double boxer_size_f64_get_height(struct SizeBoxF64 *ptr);

void boxer_size_f64_set_height(struct SizeBoxF64 *ptr, double height);

struct SizeBoxI32 *boxer_size_i32_create(void);

void boxer_size_i32_drop(struct SizeBoxI32 *ptr);

int32_t boxer_size_i32_get_width(struct SizeBoxI32 *ptr);

void boxer_size_i32_set_width(struct SizeBoxI32 *ptr, int32_t width);

int32_t boxer_size_i32_get_height(struct SizeBoxI32 *ptr);

void boxer_size_i32_set_height(struct SizeBoxI32 *ptr, int32_t height);

struct SizeBoxU32 *boxer_size_u32_create(void);

void boxer_size_u32_drop(struct SizeBoxU32 *ptr);

uint32_t boxer_size_u32_get_width(struct SizeBoxU32 *ptr);

void boxer_size_u32_set_width(struct SizeBoxU32 *ptr, uint32_t width);

uint32_t boxer_size_u32_get_height(struct SizeBoxU32 *ptr);

void boxer_size_u32_set_height(struct SizeBoxU32 *ptr, uint32_t height);

struct SizeBoxU64 *boxer_size_u64_create(void);

void boxer_size_u64_drop(struct SizeBoxU64 *ptr);

uint64_t boxer_size_u64_get_width(struct SizeBoxU64 *ptr);

void boxer_size_u64_set_width(struct SizeBoxU64 *ptr, uint64_t width);

uint64_t boxer_size_u64_get_height(struct SizeBoxU64 *ptr);

void boxer_size_u64_set_height(struct SizeBoxU64 *ptr, uint64_t height);

struct StringBox *boxer_string_create(void);

/**
 * I copy the data (must *not* contain zero-byte).
 * length must not include the zero-byte
 */
struct StringBox *boxer_string_from_byte_string(const uint8_t *data, size_t length);

/**
 * I copy the data (must *not* contain zero-byte).
 * length must not include the zero-byte
 */
struct StringBox *boxer_string_from_wide_string(const uint32_t *data, size_t length);

/**
 * I copy the data (must contain zero-byte).
 * length must not include the zero-byte
 */
struct StringBox *boxer_string_from_utf8_string(const uint8_t *data, size_t length);

void boxer_string_drop(struct StringBox *string_box);

size_t boxer_string_get_len(struct StringBox *string_box);

size_t boxer_string_get_char_count(struct StringBox *string_box);

const uint8_t *boxer_string_get_ptr(struct StringBox *string_box);

void boxer_string_char_index_to_byte_range(struct StringBox *string_ptr,
                                           size_t index,
                                           struct ValueBox_Range_usize *range_ptr);

void boxer_string_char_index_to_utf16_range(struct StringBox *string_box,
                                            size_t index,
                                            struct ValueBox_Range_usize *range_ptr);

size_t boxer_string_utf16_position_to_char_index(struct StringBox *string_box, size_t index);

const void *boxer_value_box_get_pointer(struct ValueBox_c_void *ptr);

bool boxer_value_box_is_valid(struct ValueBox_c_void *ptr);

#endif /* winit_h */
